@@ -16,36 +16,28 @@
  *  along with Phototonic.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <QtWidgets>
-#include <QtWidgets/QtWidgets>
-#include "Settings.h"
-#include "FileSystemModel.h"
-
 #ifndef FILE_SYSTEM_TREE_H
 #define FILE_SYSTEM_TREE_H
 
+class QFileSystemModel;
+#include <QTreeView>
+
+
 class FileSystemTree : public QTreeView {
 
 Q_OBJECT
 
 public:
     FileSystemTree(QWidget *parent);
-
-    FileSystemModel *fileSystemModel;
-
     QModelIndex getCurrentIndex();
-
-    void setModelFlags();
+    virtual void setModel(QAbstractItemModel *model) override;
 
 protected:
     void dragEnterEvent(QDragEnterEvent *event);
-
     void dragMoveEvent(QDragMoveEvent *event);
-
     void dropEvent(QDropEvent *event);
 
 signals:
-
     void dropOp(Qt::KeyboardModifiers keyMods, bool dirOp, QString cpMvDirPath);
 
 private:
