@@ -22,8 +22,20 @@
 #include "MessageBox.h"
 #include "Phototonic.h"
 
-MessageBox::MessageBox(QWidget *parent) : QMessageBox(parent) {
+MessageBox::MessageBox(QWidget *parent, StandardButtons buttons, StandardButton defaultButton) : QMessageBox(parent) {
     setWindowIcon(QIcon(":/images/phototonic.png"));
+    if (buttons != NoButton) {
+        setStandardButtons(buttons);
+        if (defaultButton != NoButton)
+            setDefaultButton(defaultButton);
+    }
+}
+
+int MessageBox::ask(const QString &title, const QString &message) {
+    setWindowTitle(title);
+    setText(message);
+    setIcon(MessageBox::Question);
+    return exec();
 }
 
 void MessageBox::critical(const QString &title, const QString &message) {
@@ -33,11 +45,11 @@ void MessageBox::critical(const QString &title, const QString &message) {
     exec();
 }
 
-void MessageBox::warning(const QString &title, const QString &message) {
+int MessageBox::warning(const QString &title, const QString &message) {
     setWindowTitle(title);
     setText(message);
     setIcon(MessageBox::Warning);
-    exec();
+    return exec();
 }
 
 void MessageBox::about() {
@@ -52,30 +64,45 @@ void MessageBox::about() {
                     << "Code: Thomas Lübking - some patches in 2015 ;)" // thomas.luebking@gmail.com
                     << "Bosnian: Dino Duratović" // dinomol@mail.com
                     << "Chinese: BigELK176 ≡" // <BigELK176@gmail.com>
+                    << "Chinese: wenchao-cui" // <cuzz@qq.com>
                     << "Croatian: Dino Duratović" // dinomol@mail.com
+                    << "Croatian: milotype" // <mail@milotype.de>
                     << "Czech: Pavel Fric" // pavelfric@seznam.cz
+                    << "Czech: p-bo" // <pavel.borecki@gmail.com>
                     << "Dutch: Heimen Stoffels" // <vistausss@fastmail.com>
                     << "Estonian: Priit Jõerüüt" //<lxqtwlate@joeruut.com>
+                    << "Finnish: sudoasd" // <asdwasd@meiliboxi.fi>
                     << "French: Adrien Daugabel" // adrien.d@mageialinux-online.org
                     << "French: David Geiger" // david.david@mageialinux-online.org
                     << "French: Nicolas PARLANT" // <nicolas.parlant@parhuet.fr>
+                    << "French: Oliver Chiasson" // <olivierchiasson@hotmail.fr>
                     << "French: Rémi Verschelde" // akien@mageia.org
                     << "Georgian: NorwayFun" //  <temuri.doghonadze@gmail.com>
+                    << "German: EttoreAtalan" // <atalanttore@googlemail.com>
                     << "German: Jonathan Hooverman" // jonathan.hooverman@gmail.com
                     << "German: Some wordstumbling fool" // thomas.luebking@gmail.com
+                    << "Hebrew: Yaron Shahrabani" // <sh.yaron@gmail.com>
                     << "Italian: Standreas (stefonarch)" // github.com/stefonarch
+                    << "Lithuanian: Moo" // <hazap@hotmail.com>
                     << "Polish: Jan Rolski" // <wbcwknvstb@proton.me>
                     << "Polish: Krzysztof Pyrkosz" // pyrkosz@o2.pl
                     << "Polish: Robert Wojewódzki" // robwoj44@poczta.onet.pl
                     << "Portuguese: Hugo Carvalho" // <hugokarvalho@hotmail.com>
                     << "Portuguese: Marcos M. Nascimento" // wstlmn@uol.com.br
+                    << "Russian: Azasis" // <lionovdmitrij386@gmail.com>
+                    << "Russian: Dmitry" // <dmitrydmitry761@gmail.com>
                     << "Russian: Ilya Alexandrovich" // yast4ik@gmail.com
+                    << "Russian: Valera" // <ValeraDX@yandex.ru>
                     << "Serbian: Dino Duratović" // dinomol@mail.com
-                    << "Swedish: Luna Jernberg" // <droidbittin@gmail.com>
+                    << "Spanish: Roberalz" // <lumintnian@outlook.com>
+                    << "Spanish: Megido" // <hibarioath@proton.me>
+//                    << "Swedish: Luna Jernberg" // <droidbittin@gmail.com>
+                    << "Swedish: bittin" // <bittin@reimu.nl>
+                    << "Swedish: Daniel Nylander" // <po@danielnylander.se>
                     << "Ukrainian: Ihor Hordiichuk" // <igor_ck@outlook.com>
                     << "Standreas and the entire LXQt weblate team"
                     ;
-    QString aboutString = "<h1>Phototonic 3.0</h1>" // + QString(VERSION)
+    QString aboutString = "<h1>Phototonic 3.1.0</h1>" // + QString(VERSION)
                           "<h4>" + tr("Image Viewer and Organizer") + "</h4>"
                           "<a href=\"https://github.com/luebking/phototonic\">" + tr("Home page and bug reports") + "</a>"
                           "<dl><dt>Copyright</dt><dd>"
