@@ -19,14 +19,18 @@
 #ifndef META_DATA_CACHE_H
 #define META_DATA_CACHE_H
 
+#include <QMultiMap>
 #include <QTransform>
 
 namespace Metadata {
+    typedef QMultiMap<QString, QPair<QString,QString>> DataTriple;
+    typedef QMultiMap<QString, QString> DataPair;
     bool addTag(const QString &imageFileName, const QString &tagName);
     void cache(const QString &imageFullPath);
-    void data(const QString &imageFullPath, QMap<QString,QString> *EXIF = nullptr,
-              QMap<QString,QString> *IPTC = nullptr, QMap<QString,QString> *XMP = nullptr);
+    void data(const QString &imageFullPath, DataTriple *EXIF = nullptr, DataTriple *IPTC = nullptr, DataTriple *XMP = nullptr);
+    bool setData(const QString &imageFullPath, DataPair EXIF, DataPair IPTC, DataPair XMP);
     void dropCache();
+    bool gpsData(const QString &imageFullPath, double &lat, double &lon, double &alt);
     QTransform transformation(const QString &imageFullPath);
     void forget(const QString &imageFileName);
     void rename(const QString &oldImageFileName, const QString &newImageFileName);
