@@ -59,6 +59,7 @@ namespace Settings {
     extern const char optionDefaultSaveQuality[];
     extern const char optionSlideShowDelay[];
     extern const char optionSlideShowRandom[];
+    extern const char optionSlideShowCrossfade[];
     extern const char optionFileSystemDockVisible[];
     extern const char optionBookmarksDockVisible[];
     extern const char optionImagePreviewDockVisible[];
@@ -76,6 +77,7 @@ namespace Settings {
     extern const char optionShortcuts[];
     extern const char optionExternalApps[];
     extern const char optionBangs[];
+    extern const char optionExifFilters[];
     extern const char optionWallpaperCommand[];
     extern const char optionCopyMoveToPaths[];
     extern const char optionKnownTags[];
@@ -98,12 +100,11 @@ namespace Settings {
     extern qreal rotation;
     extern bool mouseRotateEnabled;
     extern bool keepTransform;
-    extern bool flipH;
-    extern bool flipV;
     extern int defaultSaveQuality;
-    extern int slideShowDelay;
+    extern double slideShowDelay;
     extern bool slideShowRandom;
     extern bool slideShowActive;
+    extern bool slideShowCrossfade;
     extern QMap<QString, QAction *> actionKeys;
     extern int hueVal;
     extern int saturationVal;
@@ -128,6 +129,7 @@ namespace Settings {
     extern bool showViewerToolbar;
     extern QMap<QString, QString> externalApps;
     extern QMap<QString, QString> bangs;
+    extern QMap<QString, QString> exifFilters;
     extern QString wallpaperCommand;
     extern QSet<QString> bookmarkPaths;
     extern QSet<QString> knownTags;
@@ -159,6 +161,7 @@ namespace Settings {
     extern bool upscalePreview;
     extern bool scrollZooms;
     extern int dupeAccuracy;
+    extern QStringList imageToolActions;
 }
 
 #endif // SETTINGS_H
