@@ -16,55 +16,41 @@
  *  along with Phototonic.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <QBoxLayout>
+#include <QCheckBox>
+#include <QColorDialog>
+#include <QDoubleSpinBox>
+#include <QFileDialog>
+#include <QGroupBox>
+#include <QHeaderView>
+#include <QLabel>
+#include <QLineEdit>
+#include <QPainter>
+#include <QPushButton>
+#include <QRadioButton>
+#include <QSpinBox>
+#include <QTabWidget>
+#include <QTableWidget>
+#include <QToolButton>
+
+#include "Settings.h"
 #include "SettingsDialog.h"
+#include "ShortcutsTable.h"
 
 SettingsDialog::SettingsDialog(QWidget *parent) : QDialog(parent) {
     setWindowTitle(tr("Preferences"));
     setWindowIcon(QIcon::fromTheme("preferences-system", QIcon(":/images/phototonic.png")));
 
-    // Zoom large images
-    QGroupBox *fitLargeGroupBox = new QGroupBox(tr("Fit Large Images"));
-    fitLargeRadios[0] = new QRadioButton(tr("Disable"));
-    fitLargeRadios[1] = new QRadioButton(tr("By width or height"));
-    fitLargeRadios[2] = new QRadioButton(tr("By width"));
-    fitLargeRadios[3] = new QRadioButton(tr("By height"));
-    fitLargeRadios[4] = new QRadioButton(tr("Stretch disproportionately"));
-    QVBoxLayout *fitLargeVbox = new QVBoxLayout;
-    for (int i = 0; i < nZoomRadios; ++i) {
-        fitLargeVbox->addWidget(fitLargeRadios[i]);
-        fitLargeRadios[i]->setChecked(false);
-    }
-    fitLargeVbox->addStretch(1);
-    fitLargeGroupBox->setLayout(fitLargeVbox);
-    fitLargeRadios[Settings::zoomOutFlags]->setChecked(true);
-
-    // Zoom small images
-    QGroupBox *fitSmallGroupBox = new QGroupBox(tr("Fit Small Images"));
-    fitSmallRadios[0] = new QRadioButton(tr("Disable"));
-    fitSmallRadios[1] = new QRadioButton(tr("By width or height"));
-    fitSmallRadios[2] = new QRadioButton(tr("By width"));
-    fitSmallRadios[3] = new QRadioButton(tr("By height"));
-    fitSmallRadios[4] = new QRadioButton(tr("Stretch disproportionately"));
-    QVBoxLayout *fitSmallVbox = new QVBoxLayout;
-    for (int i = 0; i < nZoomRadios; ++i) {
-        fitSmallVbox->addWidget(fitSmallRadios[i]);
-        fitSmallRadios[i]->setChecked(false);
-    }
-    fitSmallVbox->addStretch(1);
-    fitSmallGroupBox->setLayout(fitSmallVbox);
-    fitSmallRadios[Settings::zoomInFlags]->setChecked(true);
-
     // imageViewer background color
     QLabel *backgroundColorLabel = new QLabel(tr("Background color:"));
     backgroundColorButton = new QToolButton();
-    backgroundColorButton->setFixedSize(48, 24);
+    backgroundColorButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
     QHBoxLayout *backgroundColorHBox = new QHBoxLayout;
     backgroundColorHBox->addWidget(backgroundColorLabel);
     backgroundColorHBox->addWidget(backgroundColorButton);
     backgroundColorHBox->addStretch(1);
     connect(backgroundColorButton, SIGNAL(clicked()), this, SLOT(pickColor()));
     setButtonBgColor(Settings::viewerBackgroundColor, backgroundColorButton);
-    backgroundColorButton->setAutoFillBackground(true);
     imageViewerBackgroundColor = Settings::viewerBackgroundColor;
 
     // Wrap image list
@@ -93,47 +79,70 @@ SettingsDialog::SettingsDialog(QWidget *parent) : QDialog(parent) {
     showImageNameCheckBox = new QCheckBox(tr("Show image file name in viewer"), this);
     showImageNameCheckBox->setChecked(Settings::showImageName);
 
+    scrollZoomCheckBox = new QCheckBox(tr("Use scroll wheel for zooming"), this);
+    scrollZoomCheckBox->setChecked(Settings::scrollZooms);
+
+    // Slide show delay
+    QLabel *slideDelayLab = new QLabel(tr("Delay between slides in seconds:"));
+    slideDelaySpinBox = new QDoubleSpinBox;
+    slideDelaySpinBox->setRange(0.1, 3600.0);
+    slideDelaySpinBox->setValue(Settings::slideShowDelay);
+    QHBoxLayout *slideDelayLayout = new QHBoxLayout;
+    slideDelayLayout->addWidget(slideDelayLab);
+    slideDelayLayout->addWidget(slideDelaySpinBox);
+    slideDelayLayout->addStretch(1);
+
+    // Slide show random
+    slideRandomCheckBox = new QCheckBox(tr("Show random images"), this);
+    slideRandomCheckBox->setChecked(Settings::slideShowRandom);
+
+    // Slide show random
+    slideCrossfadeCheckBox = new QCheckBox(tr("Crossfade images"), this);
+    slideCrossfadeCheckBox->setChecked(Settings::slideShowCrossfade);
+    connect(slideDelaySpinBox, &QDoubleSpinBox::valueChanged, [=](double d){ slideCrossfadeCheckBox->setEnabled(d >= 1.0);});
+
+    // Slide show options
+    QGroupBox *slideshowGroupBox = new QGroupBox(tr("Slideshow"));
+    QVBoxLayout *slideshowLayout = new QVBoxLayout(slideshowGroupBox);
+    slideshowLayout->addLayout(slideDelayLayout);
+    slideshowLayout->addWidget(slideRandomCheckBox);
+    slideshowLayout->addWidget(slideCrossfadeCheckBox);
+    slideshowLayout->addStretch(1);
+
     // Viewer options
     QVBoxLayout *viewerOptsBox = new QVBoxLayout;
-    QHBoxLayout *zoomOptsBox = new QHBoxLayout;
-    zoomOptsBox->setAlignment(Qt::AlignTop);
-    zoomOptsBox->addWidget(fitLargeGroupBox);
-    zoomOptsBox->addWidget(fitSmallGroupBox);
-    zoomOptsBox->addStretch(1);
-
-    viewerOptsBox->addLayout(zoomOptsBox);
     viewerOptsBox->addLayout(backgroundColorHBox);
     viewerOptsBox->addWidget(enableExifCheckBox);
-    viewerOptsBox->addWidget(showImageNameCheckBox);
-    viewerOptsBox->addWidget(wrapListCheckBox);
     viewerOptsBox->addWidget(enableAnimCheckBox);
+    viewerOptsBox->addWidget(showImageNameCheckBox);
+    viewerOptsBox->addWidget(scrollZoomCheckBox);
+    viewerOptsBox->addWidget(wrapListCheckBox);
     viewerOptsBox->addLayout(saveQualityHbox);
+    viewerOptsBox->addWidget(slideshowGroupBox);
     viewerOptsBox->addStretch(1);
 
     // thumbsViewer background color
     QLabel *thumbsBackgroundColorLabel = new QLabel(tr("Thumbnails and Preview Background Color:"));
     thumbsColorPickerButton = new QToolButton();
-    thumbsColorPickerButton->setFixedSize(48, 24);
+    thumbsColorPickerButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
     QHBoxLayout *thumbsBackgroundColorLayout = new QHBoxLayout;
     thumbsBackgroundColorLayout->addWidget(thumbsBackgroundColorLabel);
     thumbsBackgroundColorLayout->addWidget(thumbsColorPickerButton);
     thumbsBackgroundColorLayout->addStretch(1);
     connect(thumbsColorPickerButton, SIGNAL(clicked()), this, SLOT(pickThumbsColor()));
     setButtonBgColor(Settings::thumbsBackgroundColor, thumbsColorPickerButton);
-    thumbsColorPickerButton->setAutoFillBackground(true);
     thumbsBackgroundColor = Settings::thumbsBackgroundColor;
 
     // thumbsViewer text color
     QLabel *thumbLabelColorLabel = new QLabel(tr("Label color:"));
     thumbsLabelColorButton = new QToolButton();
-    thumbsLabelColorButton->setFixedSize(48, 24);
+    thumbsLabelColorButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
     QHBoxLayout *thumbsLabelColorLayout = new QHBoxLayout;
     thumbsLabelColorLayout->addWidget(thumbLabelColorLabel);
     thumbsLabelColorLayout->addWidget(thumbsLabelColorButton);
     thumbsLabelColorLayout->addStretch(1);
     connect(thumbsLabelColorButton, SIGNAL(clicked()), this, SLOT(pickThumbsTextColor()));
     setButtonBgColor(Settings::thumbsTextColor, thumbsLabelColorButton);
-    thumbsLabelColorButton->setAutoFillBackground(true);
     thumbsTextColor = Settings::thumbsTextColor;
 
     // thumbsViewer background image
@@ -142,12 +151,14 @@ SettingsDialog::SettingsDialog(QWidget *parent) : QDialog(parent) {
     thumbsBackgroundImageLineEdit->setClearButtonEnabled(true);
     thumbsBackgroundImageLineEdit->setMinimumWidth(200);
 
-    QToolButton *chooseThumbsBackImageButton = new QToolButton();
+
+    QPushButton *chooseThumbsBackImageButton = new QPushButton();
     chooseThumbsBackImageButton->setIcon(QIcon::fromTheme("document-open", QIcon(":/images/open.png")));
     chooseThumbsBackImageButton->setFixedSize(26, 26);
     chooseThumbsBackImageButton->setIconSize(QSize(16, 16));
     connect(chooseThumbsBackImageButton, SIGNAL(clicked()), this, SLOT(pickBackgroundImage()));
 
+
     QHBoxLayout *thumbsBackgroundImageLayout = new QHBoxLayout;
     thumbsBackgroundImageLayout->addWidget(thumbsBackgroundImageLabel);
     thumbsBackgroundImageLayout->addWidget(thumbsBackgroundImageLineEdit);
@@ -168,13 +179,19 @@ SettingsDialog::SettingsDialog(QWidget *parent) : QDialog(parent) {
     enableThumbExifCheckBox = new QCheckBox(tr("Rotate thumbnail according to Exif orientation value"), this);
     enableThumbExifCheckBox->setChecked(Settings::exifThumbRotationEnabled);
 
+    // Upscale preview image
+    upscalePreviewCheckBox = new QCheckBox(tr("Scale up small images in preview"), this);
+    upscalePreviewCheckBox->setChecked(Settings::upscalePreview);
+
     // Thumbnail options
     QVBoxLayout *thumbsOptsBox = new QVBoxLayout;
     thumbsOptsBox->addLayout(thumbsBackgroundColorLayout);
 
     thumbsOptsBox->addLayout(thumbsBackgroundImageLayout);
+
     thumbsOptsBox->addLayout(thumbsLabelColorLayout);
     thumbsOptsBox->addWidget(enableThumbExifCheckBox);
+    thumbsOptsBox->addWidget(upscalePreviewCheckBox);
     thumbsOptsBox->addLayout(thumbPagesReadLayout);
     thumbsOptsBox->addStretch(1);
 
@@ -190,6 +207,7 @@ SettingsDialog::SettingsDialog(QWidget *parent) : QDialog(parent) {
     QGroupBox *startupDirGroupBox = new QGroupBox(tr("Startup directory if not specified by command line"));
     startupDirectoryRadioButtons[Settings::RememberLastDir] = new QRadioButton(tr("Remember last"));
     startupDirectoryRadioButtons[Settings::DefaultDir] = new QRadioButton(tr("Default"));
+    //: specificy a startup directory
     startupDirectoryRadioButtons[Settings::SpecifiedDir] = new QRadioButton(tr("Specify:"));
 
     startupDirLineEdit = new QLineEdit;
@@ -197,7 +215,7 @@ SettingsDialog::SettingsDialog(QWidget *parent) : QDialog(parent) {
     startupDirLineEdit->setMinimumWidth(300);
     startupDirLineEdit->setMaximumWidth(400);
 
-    QToolButton *chooseStartupDirButton = new QToolButton();
+    QPushButton *chooseStartupDirButton = new QPushButton();
     chooseStartupDirButton->setIcon(QIcon::fromTheme("document-open", QIcon(":/images/open.png")));
     chooseStartupDirButton->setFixedSize(26, 26);
     chooseStartupDirButton->setIconSize(QSize(16, 16));
@@ -230,7 +248,7 @@ SettingsDialog::SettingsDialog(QWidget *parent) : QDialog(parent) {
     // Keyboard shortcuts
     ShortcutsTable *shortcutsTable = new ShortcutsTable();
     shortcutsTable->refreshShortcuts();
-    QGroupBox *keyboardGroupBox = new QGroupBox(tr("Keyboard"));
+    QGroupBox *keyboardGroupBox = new QGroupBox(tr("Shortcuts"));
     QVBoxLayout *keyboardSettingsLayout = new QVBoxLayout;
 
     QHBoxLayout *filterShortcutsLayout = new QHBoxLayout;
@@ -255,32 +273,76 @@ SettingsDialog::SettingsDialog(QWidget *parent) : QDialog(parent) {
     generalSettingsLayout->addWidget(deleteConfirmCheckBox);
     generalSettingsLayout->addWidget(startupDirGroupBox);
 
-    // Slide show delay
-    QLabel *slideDelayLab = new QLabel(tr("Delay between slides in seconds:"));
-    slideDelaySpinBox = new QSpinBox;
-    slideDelaySpinBox->setRange(1, 3600);
-    slideDelaySpinBox->setValue(Settings::slideShowDelay);
-    QHBoxLayout *slideDelayLayout = new QHBoxLayout;
-    slideDelayLayout->addWidget(slideDelayLab);
-    slideDelayLayout->addWidget(slideDelaySpinBox);
-    slideDelayLayout->addStretch(1);
-
-    // Slide show random
-    slideRandomCheckBox = new QCheckBox(tr("Show random images"), this);
-    slideRandomCheckBox->setChecked(Settings::slideShowRandom);
-
-    // Slide show options
-    QVBoxLayout *slideshowLayout = new QVBoxLayout;
-    slideshowLayout->addLayout(slideDelayLayout);
-    slideshowLayout->addWidget(slideRandomCheckBox);
-    slideshowLayout->addStretch(1);
-
-    QGroupBox *slideshowGroupBox = new QGroupBox(tr("Slide Show"));
-    slideshowGroupBox->setLayout(slideshowLayout);
-    generalSettingsLayout->addWidget(slideshowGroupBox);
     generalSettingsLayout->addWidget(setWindowIconCheckBox);
     generalSettingsLayout->addStretch(1);
 
+    QVBoxLayout *bangLayout = new QVBoxLayout;
+    QLabel *rtfm = new QLabel(
+    tr("<p>Bangs allow you to use external commands to generate the list of shown images.<br>"
+    "The main purpose is to query databases like locate, baloo or tracker, "
+    "but anything that can generate a list of image files is suitable</p>"
+    "<p>The token <b>%s</b> in the command will be replaced with the parameter.</p>"
+    "<p>Eg. for plocate, using the shortcut <i>locate</i> and the command<br>"
+    "<i>bash -c \"locate -i '*%s*' | grep --line-buffered -iE '(jpe?g|png)$'\"</i><br>"
+    "allows you to enter <i>locate:waldo</i> to display indexed jpg's and png's of waldo.</p>"
+    "<p>Phototonic tests the files for existence and will remove duplicates (including file and "
+    "directory symlinks.</p>"));
+    rtfm->setWordWrap(true);
+    bangLayout->addWidget(rtfm);
+    bangTable = new QTableWidget(this);
+    bangTable->setColumnCount(2);
+    bangTable->setSelectionMode(QAbstractItemView::SingleSelection);
+    bangTable->setSelectionBehavior(QAbstractItemView::SelectItems);
+    bangTable->verticalHeader()->setVisible(false);
+    bangTable->verticalHeader()->setDefaultSectionSize(bangTable->verticalHeader()->minimumSectionSize());
+    bangTable->setHorizontalHeaderItem(0, new QTableWidgetItem(tr("Shortcut")));
+    bangTable->setHorizontalHeaderItem(1, new QTableWidgetItem(tr("Command")));
+    bangTable->horizontalHeader()->setSectionResizeMode(0, QHeaderView::Interactive);
+    bangTable->horizontalHeader()->setSectionResizeMode(1, QHeaderView::Stretch);
+    bangTable->setShowGrid(true);
+    bangTable->setRowCount(Settings::bangs.size()+1);
+    int row = 0;
+    for (auto i = Settings::bangs.cbegin(), end = Settings::bangs.cend(); i != end; ++i) {
+        bangTable->setItem(row, 0, new QTableWidgetItem(i.key()));
+        bangTable->setItem(row++, 1, new QTableWidgetItem(i.value()));
+    }
+    bangLayout->addWidget(bangTable);
+    connect (bangTable, &QTableWidget::cellChanged, this, [=](int row, int col) {
+        if (row < bangTable->rowCount() - 1 && bangTable->item(row,0)->text().isEmpty() && bangTable->item(row,1)->text().isEmpty()) {
+            bangTable->removeRow(row);
+        } else {
+            if (col == 0) {
+                bangTable->blockSignals(true);
+                for (int i = 0; i < bangTable->rowCount(); ++i) {
+                    if (QTableWidgetItem *item = bangTable->item(i, 0))
+                        item->setForeground(QBrush());
+                }
+                for (int i = 0; i < bangTable->rowCount() - 1; ++i) {
+                    QTableWidgetItem *item_i = bangTable->item(i, 0);
+                    if (!item_i)
+                        continue;
+                for (int j = i + 1; j < bangTable->rowCount(); ++j) {
+                    QTableWidgetItem *item_j = bangTable->item(j, 0);
+                    if (!item_j || item_j->text() != item_i->text())
+                        continue;
+                    item_i->setForeground(QColor(208,23,23));
+                    item_j->setForeground(QColor(208,23,23));
+                }
+                }
+                bangTable->blockSignals(false);
+            }
+            if (row == bangTable->rowCount()-1 && !bangTable->item(row,col)->text().isEmpty())
+                bangTable->setRowCount(bangTable->rowCount() + 1);
+        }
+    });
+    QAction *act = new QAction;
+    act->setShortcut(Qt::Key_Delete);
+    act->setShortcutContext(Qt::WidgetShortcut);
+    connect(act, &QAction::triggered, [=]() {
+        bangTable->currentItem()->setText(QString());
+    });
+    bangTable->addAction(act);
+
     /* Confirmation buttons */
     QHBoxLayout *confirmSettingsLayout = new QHBoxLayout;
     QPushButton *okButton = new QPushButton(tr("OK"));
@@ -310,7 +372,11 @@ SettingsDialog::SettingsDialog(QWidget *parent) : QDialog(parent) {
 
     QWidget *keyboardSettings = new QWidget;
     keyboardSettings->setLayout(keyboardSettingsLayout);
-    settingsTabs->addTab(keyboardSettings, tr("Keyboard"));
+    settingsTabs->addTab(keyboardSettings, tr("Shortcuts"));
+
+    QWidget *bangSettings = new QWidget;
+    bangSettings->setLayout(bangLayout);
+    settingsTabs->addTab(bangSettings, tr("Bangs"));
 
     QVBoxLayout *mainLayout = new QVBoxLayout;
     mainLayout->addWidget(settingsTabs);
@@ -319,24 +385,6 @@ SettingsDialog::SettingsDialog(QWidget *parent) : QDialog(parent) {
 }
 
 void SettingsDialog::saveSettings() {
-    unsigned int i;
-
-    for (i = 0; i < nZoomRadios; ++i) {
-        if (fitLargeRadios[i]->isChecked()) {
-            Settings::zoomOutFlags = i;
-            Settings::appSettings->setValue(Settings::optionViewerZoomOutFlags, (int) Settings::zoomOutFlags);
-            break;
-        }
-    }
-
-    for (i = 0; i < nZoomRadios; ++i) {
-        if (fitSmallRadios[i]->isChecked()) {
-            Settings::zoomInFlags = i;
-            Settings::appSettings->setValue(Settings::optionViewerZoomInFlags, (int) Settings::zoomInFlags);
-            break;
-        }
-    }
-
     Settings::viewerBackgroundColor = imageViewerBackgroundColor;
     Settings::thumbsBackgroundColor = thumbsBackgroundColor;
     Settings::thumbsTextColor = thumbsTextColor;
@@ -346,13 +394,16 @@ void SettingsDialog::saveSettings() {
     Settings::defaultSaveQuality = saveQualitySpinBox->value();
     Settings::slideShowDelay = slideDelaySpinBox->value();
     Settings::slideShowRandom = slideRandomCheckBox->isChecked();
+    Settings::slideShowCrossfade = slideCrossfadeCheckBox->isChecked();
     Settings::enableAnimations = enableAnimCheckBox->isChecked();
     Settings::exifRotationEnabled = enableExifCheckBox->isChecked();
     Settings::exifThumbRotationEnabled = enableThumbExifCheckBox->isChecked();
     Settings::showImageName = showImageNameCheckBox->isChecked();
     Settings::reverseMouseBehavior = reverseMouseCheckBox->isChecked();
+    Settings::scrollZooms = scrollZoomCheckBox->isChecked();
     Settings::deleteConfirm = deleteConfirmCheckBox->isChecked();
     Settings::setWindowIcon = setWindowIconCheckBox->isChecked();
+    Settings::upscalePreview = upscalePreviewCheckBox->isChecked();
 
     if (startupDirectoryRadioButtons[Settings::RememberLastDir]->isChecked()) {
         Settings::startupDir = Settings::RememberLastDir;
@@ -363,6 +414,17 @@ void SettingsDialog::saveSettings() {
         Settings::specifiedStartDir = startupDirLineEdit->text();
     }
 
+    Settings::bangs.clear();
+    for (int i = 0; i < bangTable->rowCount() - 1; ++i) {
+        QString key = QString("bang_%1").arg(i);
+        if (bangTable->item(i,0) && !bangTable->item(i,0)->text().isEmpty())
+            key = bangTable->item(i,0)->text();
+        QString value;
+        if (bangTable->item(i,1))
+            value = bangTable->item(i,1)->text();
+        Settings::bangs[key] = value;
+    }
+
     accept();
 }
 
@@ -378,14 +440,21 @@ void SettingsDialog::pickColor() {
     }
 }
 
-void SettingsDialog::setButtonBgColor(QColor &color, QToolButton *button) {
-    QString style = "background: rgb(%1, %2, %3);";
-    style = style.arg(color.red()).arg(color.green()).arg(color.blue());
-    button->setStyleSheet(style);
+void SettingsDialog::setButtonBgColor(QColor &color, QAbstractButton *button) {
+    const int s = qMin(button->width(),button->height());
+    QPixmap pix(s, s);
+    pix.fill(Qt::transparent);
+    QPainter p(&pix);
+    p.setRenderHint(QPainter::Antialiasing);
+    p.setBrush(color);
+    p.drawEllipse(pix.rect());
+    p.end();
+    button->setIcon(pix);
+    button->setText(color.name());
 }
 
 void SettingsDialog::pickThumbsColor() {
-    QColor userColor = QColorDialog::getColor(Settings::thumbsBackgroundColor, this);
+    QColor userColor = QColorDialog::getColor(Settings::thumbsBackgroundColor, this, tr("Select background color"), QColorDialog::ShowAlphaChannel);
     if (userColor.isValid()) {
         setButtonBgColor(userColor, thumbsColorPickerButton);
         thumbsBackgroundColor = userColor;
